﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using System.Diagnostics;
using Infragistics.Windows;

namespace TreeView
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            var Data = new ObservableCollection<Level>();

            for (int i = 0; i < 10; i++)
            {
                Level lvl = new Level
                {
                    Id = i,
                    Name = "Level0 - " + i,
                    Date = DateTime.Now.AddDays(i),
                    Type = i % 2 == 0 ? "Type " + i : "",
                    Description = i % 2 != 0 ? "Description " + i : "",
                    ChildLevel = new ObservableCollection<Level>()
                };

                for (int j = 0; j < 5; j++)
                {
                    Level lvl1 = new Level
                    {
                        Name = "Lavel1 - " + j,
                        Date = DateTime.Now.AddYears(j),
                        Type = "",
                        Description = "Level1 Description",
                        ChildLevel = new ObservableCollection<Level>()
                    };

                    for (int p = 0; p < 5; p++)
                    {
                        Level lvl2 = new Level
                        {
                            Name = "Level2 - " + p,
                            Date = DateTime.Now.AddYears(p),
                            Type = "Some Type" + p,
                            ChildLevel = new ObservableCollection<Level>()
                        };

                        for (int q = 0; q < 5; q++)
                        {
                            lvl2.ChildLevel.Add(new Level { Name = "Level3 - " + q, Date = DateTime.Now.AddDays(q), Type = "", Description = "Text" });
                        }

                        lvl1.ChildLevel.Add(lvl2);
                    }

                    lvl.ChildLevel.Add(lvl1);
                }

                Data.Add(lvl);

                DataContext = Data;
            }
        }

        #region Events
        List<LabelPresenter> presenters = new List<LabelPresenter>();
        void Label_Loaded(object sender, RoutedEventArgs e)
        {
            LabelPresenter lp = sender as LabelPresenter;

            int currentDepth = Convert.ToInt32(lp.Field.Owner.Key.ToString().LastOrDefault().ToString());
            if (currentDepth > 0)
                lp.Height = 0;

            XamDataGrid xamDataGrid1 = lp.Field.Owner.DataPresenter as XamDataGrid;

            if (lp.Field.Index == 0)
            {
                double width = lp.Field.CellWidthResolved;
                int index = FieldLayoutIndexFromField(lp.Field, xamDataGrid1);

                if (index > -1 && lp.Field.Width == null)
                {
                    double final = width + 17 * (2 - index);
                    lp.Field.Width = new FieldLength(final);
                }
            }
        }


     

        void Record_Loaded(object sender, RoutedEventArgs e)
        {
            if ((sender as DataRecordPresenter).IsHeaderRecord)
            {
                int currentDepth = Convert.ToInt32((sender as DataRecordPresenter).Record.FieldLayout.Key.ToString().LastOrDefault().ToString());
                if (currentDepth > 0)
                {
                    Grid g = Utilities.GetDescendantFromType(sender as DependencyObject, typeof(Grid), false) as Grid;
                    g.Height = 0;
                }
            }
        }

        private void XamDataGrid_AssigningFieldLayoutToItem(object sender, Infragistics.Windows.DataPresenter.Events.AssigningFieldLayoutToItemEventArgs e)
        {
            if (e.ParentExpandableFieldRecord == null)
            {
                e.FieldLayout = e.FieldLayout.DataPresenter.FieldLayouts["lvl0"];
                e.FieldLayout.Fields.CollectionChanged += Fields_CollectionChanged;
            }
            if (e.ParentExpandableFieldRecord != null)
            {
                int index = (e.ParentExpandableFieldRecord.NestingDepth / 2) + 1;

                FieldLayout fl = e.FieldLayout.DataPresenter.FieldLayouts.Where(f => f.Key != null && f.Key.Equals("lvl" + index)).FirstOrDefault();
                if (fl == null)
                {
                    fl = new FieldLayout { Key = "lvl" + index };
                    fl.Fields.CollectionChanged += Fields_CollectionChanged;
                    e.FieldLayout.DataPresenter.FieldLayouts.Add(fl);
                }

                e.FieldLayout = fl;
            }

        }

        void Fields_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add)
            {
                (e.NewItems[0] as Field).PropertyChanged += MainWindow_PropertyChanged;
            }

            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove)
            {
                (e.OldItems[0] as Field).PropertyChanged += MainWindow_PropertyChanged;
            }
        }

        void MainWindow_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "LabelWidthResolved")
            {
                Dispatcher.BeginInvoke(new Action(() =>
                {
                    Field field = sender as Field;

                    XamDataGrid xamDataGrid1 = field.Owner.DataPresenter as XamDataGrid;

                    //var orderedLayouts = xamDataGrid1.FieldLayouts.OrderBy(fl => Convert.ToInt32(fl.Key.ToString().LastOrDefault().ToString())).ToList();
                    int currentDepth = Convert.ToInt32(field.Owner.Key.ToString().LastOrDefault().ToString());

                    int layoutIndex = FieldLayoutIndexFromField(field, xamDataGrid1);

                    if (field.Index == 0)
                    {
                        var lowerLayouts = xamDataGrid1.FieldLayouts.Where(fl => Convert.ToInt32(fl.Key.ToString().LastOrDefault().ToString()) > currentDepth).OrderBy(fll => Convert.ToInt32(fll.Key.ToString().LastOrDefault().ToString())).ToList();

                        for (int i = 0; i < lowerLayouts.Count; i++)
                        {
                            FieldLayout layout = lowerLayouts[i];
                            int depth = Convert.ToInt32(layout.Key.ToString().LastOrDefault().ToString());
                            layout.Fields[0].Width = new FieldLength(Math.Abs(field.CellWidthResolved - ((i + 1) * 17)));
                             
                        }

                        var higherLayouts = xamDataGrid1.FieldLayouts.Where(fl => Convert.ToInt32(fl.Key.ToString().LastOrDefault().ToString()) < currentDepth).OrderByDescending(fll => Convert.ToInt32(fll.Key.ToString().LastOrDefault().ToString())).ToList();

                        for (int i = 0; i < higherLayouts.Count; i++)
                        {
                            FieldLayout layout = higherLayouts[i];
                            int depth = Convert.ToInt32(layout.Key.ToString().LastOrDefault().ToString());
                            layout.Fields[0].Width = new FieldLength(Math.Abs(field.CellWidthResolved + ((i + 1) * 17)));
                        }                       
                    }
                    else
                    {
                        for (int i = 0; i < xamDataGrid1.FieldLayouts.Count; i++)
                        {
                            if (i != layoutIndex)
                            {
                                Field f = xamDataGrid1.FieldLayouts[i].Fields[field.Index];
                                f.Width = new FieldLength(field.CellWidthResolved);
                            }
                        }
                    }

                }), System.Windows.Threading.DispatcherPriority.Background, null);
            }
            
        }

        #endregion


        #region Helpers


        public int FieldLayoutIndexFromField(Field f, XamDataGrid grid)
        {
            int index = -1;

            for (int i = 0; i < grid.FieldLayouts.Count; i++)
            {
                if (grid.FieldLayouts[i].Fields.Contains(f))
                {
                    index = i;
                    break;
                }
            }

            return index;
        }

        #endregion
    }


    public class Level
    {
        public int Id { get; set; }
        public string Name { get; set; }
        public DateTime Date { get; set; }
        public string Description { get; set; }
        public string Type { get; set; }
        public ObservableCollection<Level> ChildLevel { get; set; }
    }
}
